#import "NoRetain.h"
#import <objc/runtime.h>
#import "MultivaluedDictionary.h"

// nice way for classes to make sure they have a bean
#define NeedProperty(theprop) \
	if (self.theprop == nil) \
		[NSException raise:@"MissingProperty" format:@"property %s of class %@ not set", #theprop, [self classname]];

#define BeanSourceClass(classname) \
+ (void) initialize {\
[Beans initialiseBeanSourceClass:[classname class]];\
}

#define BeanTargetClass(classname) \
+ (void) initialize {\
[Beans initialiseBeanTargetClass:[classname class]];\
}

#define BeanClass(classname) \
+ (void) initialize {\
[Beans initialiseBeanSourceClass:[classname class]];\
[Beans initialiseBeanTargetClass:[classname class]];\
}

@interface NSObject (BeanTarget)
- (void) validateBean;
@end

@interface Beans : NSObject {
	@private
	NoRetainArray *targets;
	NoRetainDictionary *namedBeans;
	MultivaluedDictionary *addedSelectorsByClass;
	NSMutableSet * _sourceClasses;
}
@property (retain) NoRetainDictionary *namedBeans;
@property (retain) NoRetainArray *targets;
@property (retain) NSMutableSet *sourceClasses;

+ (Beans*) instance;
+ (void) initialiseBeanSourceClass:(Class) klass;
+ (void) initialiseBeanTargetClass:(Class) klass;

- (void) setNamedBean:(id) bean called:(NSString*) name;
- (void) removeBean:(id) bean  klass:(Class) klass;
- (id) beanWithName:(NSString*) name;
- (void) validateBeans;

@end