#import "Beans.h"
#import "Requester.h"
#import "Callbacks.h"
#import "FileUpload.h"
#import "HasLifecycle.h"

@class UrlLoader;

@interface UrlLoader : NSObject <HasLifecycleWithProgress,HasLifecycleWithCancel> {
	NSMutableData *data;
	NSURLConnection *conn;
	NSMutableURLRequest *request;
	NSURLResponse *urlResponse; // some requests aren't http ones?
	NSHTTPURLResponse *httpResponse;
	BOOL downloadToFile;
	NSString *file;
	NSString * _fileName;
	FILE *filePointer;
	int expectedLength;
	int bytesReceivedSoFar;
	FileUpload *fileUpload;
	NSMutableArray *successListeners;
	NSMutableArray * _finallyListeners;
	NSMutableArray * _progressListeners;
	BOOL cancelled;
	BOOL  _finished;
	NSError * _error;
	NSDate * _lastProgressNotification;
	id _customData;
}

// inputs
@property (assign) BOOL downloadToFile; // callback will receive filename rather than data - the caller must remove the file
@property (retain) NSString *fileName; // set this to make sure the downloaded file has this name (it's put in a new temp folder) 
@property (assign) NSURL *url; // use this as an alternative to building your own request
@property (retain) NSMutableURLRequest *request;
@property (retain) FileUpload *fileUpload;

// Callbacks (others defined in HasLifecycle). These will be called on the main thread.
- (void) addSuccessListener:(BasicBlock) f;

// read these to get output
@property (assign) int bytesReceivedSoFar;
@property (assign) int expectedLength;
@property (assign) BOOL finished;
@property (readonly) BOOL ok;
@property (assign) BOOL cancelled;
@property (retain) NSMutableData *data;
@property (readonly) NSString *stringResponse; // convenience - will read from data or file
@property (retain) NSString *file;
@property (readonly) NSHTTPURLResponse *httpResponse;
@property (retain) NSURLResponse *urlResponse;
@property (retain) NSError *error; // see category below

+ (UrlLoader*) loader;
+ (UrlLoader*) loaderWithUrl:(NSURL*) url;
- (void) startAndFinally:(BasicBlock) f;
- (void) startAndOnSuccess:(BasicBlock) f;
- (void) start;
// Block as long as not finished or interrupted (if interrupted, an exception will be thrown)
- (void) waitUntilFinished;


// ---- protected - for use by subclasses ----
- (void) doStart;
- (void) whenFinished;
- (void) doFinish;
- (BOOL) reset;

@end


@interface NSError (UrlLoader)
- (NSNumber*) badUrlLoaderResponseCode;
@end


